package jdk.internal.org.objectweb.asm.util;

import java.io.PrintWriter;
import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Attribute;
import jdk.internal.org.objectweb.asm.TypePath;
import java.util.ArrayList;
import java.util.List;

public abstract class Printer
{
    public static final String[] OPCODES;
    public static final String[] TYPES;
    public static final String[] HANDLE_TAG;
    protected final int api;
    protected final StringBuffer buf;
    public final List<Object> text;
    
    protected Printer(int api) {
        this.api = api;
        this.buf = new StringBuffer();
        this.text = new ArrayList();
    }
    
    public abstract void visit(int p0, int p1, String p2, String p3, String p4, String[] p5);
    
    public abstract void visitSource(String p0, String p1);
    
    public abstract void visitOuterClass(String p0, String p1, String p2);
    
    public abstract Printer visitClassAnnotation(String p0, boolean p1);
    
    public Printer visitClassTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract void visitClassAttribute(Attribute p0);
    
    public abstract void visitInnerClass(String p0, String p1, String p2, int p3);
    
    public abstract Printer visitField(int p0, String p1, String p2, String p3, Object p4);
    
    public abstract Printer visitMethod(int p0, String p1, String p2, String p3, String[] p4);
    
    public abstract void visitClassEnd();
    
    public abstract void visit(String p0, Object p1);
    
    public abstract void visitEnum(String p0, String p1, String p2);
    
    public abstract Printer visitAnnotation(String p0, String p1);
    
    public abstract Printer visitArray(String p0);
    
    public abstract void visitAnnotationEnd();
    
    public abstract Printer visitFieldAnnotation(String p0, boolean p1);
    
    public Printer visitFieldTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract void visitFieldAttribute(Attribute p0);
    
    public abstract void visitFieldEnd();
    
    public void visitParameter(String s, int n) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract Printer visitAnnotationDefault();
    
    public abstract Printer visitMethodAnnotation(String p0, boolean p1);
    
    public Printer visitMethodTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract Printer visitParameterAnnotation(int p0, String p1, boolean p2);
    
    public abstract void visitMethodAttribute(Attribute p0);
    
    public abstract void visitCode();
    
    public abstract void visitFrame(int p0, int p1, Object[] p2, int p3, Object[] p4);
    
    public abstract void visitInsn(int p0);
    
    public abstract void visitIntInsn(int p0, int p1);
    
    public abstract void visitVarInsn(int p0, int p1);
    
    public abstract void visitTypeInsn(int p0, String p1);
    
    public abstract void visitFieldInsn(int p0, String p1, String p2, String p3);
    
    @Deprecated
    public void visitMethodInsn(int n, String s, String s2, String s3) {
        if (this.api >= 327680) {
            this.visitMethodInsn(n, s, s2, s3, n == 185);
            return;
        }
        throw new RuntimeException("Must be overriden");
    }
    
    public void visitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        if (this.api < 327680) {
            if (b != (n == 185)) {
                throw new IllegalArgumentException("INVOKESPECIAL/STATIC on interfaces require ASM 5");
            }
            this.visitMethodInsn(n, s, s2, s3);
            return;
        }
        else {
            throw new RuntimeException("Must be overriden");
        }
    }
    
    public abstract void visitInvokeDynamicInsn(String p0, String p1, Handle p2, Object... p3);
    
    public abstract void visitJumpInsn(int p0, Label p1);
    
    public abstract void visitLabel(Label p0);
    
    public abstract void visitLdcInsn(Object p0);
    
    public abstract void visitIincInsn(int p0, int p1);
    
    public abstract void visitTableSwitchInsn(int p0, int p1, Label p2, Label... p3);
    
    public abstract void visitLookupSwitchInsn(Label p0, int[] p1, Label[] p2);
    
    public abstract void visitMultiANewArrayInsn(String p0, int p1);
    
    public Printer visitInsnAnnotation(int n, TypePath typePath, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract void visitTryCatchBlock(Label p0, Label p1, Label p2, String p3);
    
    public Printer visitTryCatchAnnotation(int n, TypePath typePath, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract void visitLocalVariable(String p0, String p1, String p2, Label p3, Label p4, int p5);
    
    public Printer visitLocalVariableAnnotation(int n, TypePath typePath, Label[] array, Label[] array2, int[] array3, String s, boolean b) {
        throw new RuntimeException("Must be overriden");
    }
    
    public abstract void visitLineNumber(int p0, Label p1);
    
    public abstract void visitMaxs(int p0, int p1);
    
    public abstract void visitMethodEnd();
    
    public List<Object> getText() {
        return this.text;
    }
    
    public void print(PrintWriter printWriter) {
        printList(printWriter, this.text);
    }
    
    public static void appendString(StringBuffer sb, String s) {
        sb.append('\"');
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 == '\n') {
                sb.append("\\n");
            }
            else if (char1 == '\r') {
                sb.append("\\r");
            }
            else if (char1 == '\\') {
                sb.append("\\\\");
            }
            else if (char1 == '\"') {
                sb.append("\\\"");
            }
            else if (char1 < ' ' || char1 > '\u007f') {
                sb.append("\\u");
                if (char1 < '\u0010') {
                    sb.append("000");
                }
                else if (char1 < 'Ā') {
                    sb.append("00");
                }
                else if (char1 < 'က') {
                    sb.append('0');
                }
                sb.append(Integer.toString(char1, 16));
            }
            else {
                sb.append((char)char1);
            }
        }
        sb.append('\"');
    }
    
    static void printList(PrintWriter printWriter, List<?> list) {
        for (int i = 0; i < list.size(); ++i) {
            Object value = list.get(i);
            if ((value instanceof List)) {
                printList(printWriter, (List)value);
            }
            else {
                printWriter.print(value.toString());
            }
        }
    }
    
    static {
        String s = "NOP,ACONST_NULL,ICONST_M1,ICONST_0,ICONST_1,ICONST_2,ICONST_3,ICONST_4,ICONST_5,LCONST_0,LCONST_1,FCONST_0,FCONST_1,FCONST_2,DCONST_0,DCONST_1,BIPUSH,SIPUSH,LDC,,,ILOAD,LLOAD,FLOAD,DLOAD,ALOAD,,,,,,,,,,,,,,,,,,,,,IALOAD,LALOAD,FALOAD,DALOAD,AALOAD,BALOAD,CALOAD,SALOAD,ISTORE,LSTORE,FSTORE,DSTORE,ASTORE,,,,,,,,,,,,,,,,,,,,,IASTORE,LASTORE,FASTORE,DASTORE,AASTORE,BASTORE,CASTORE,SASTORE,POP,POP2,DUP,DUP_X1,DUP_X2,DUP2,DUP2_X1,DUP2_X2,SWAP,IADD,LADD,FADD,DADD,ISUB,LSUB,FSUB,DSUB,IMUL,LMUL,FMUL,DMUL,IDIV,LDIV,FDIV,DDIV,IREM,LREM,FREM,DREM,INEG,LNEG,FNEG,DNEG,ISHL,LSHL,ISHR,LSHR,IUSHR,LUSHR,IAND,LAND,IOR,LOR,IXOR,LXOR,IINC,I2L,I2F,I2D,L2I,L2F,L2D,F2I,F2L,F2D,D2I,D2L,D2F,I2B,I2C,I2S,LCMP,FCMPL,FCMPG,DCMPL,DCMPG,IFEQ,IFNE,IFLT,IFGE,IFGT,IFLE,IF_ICMPEQ,IF_ICMPNE,IF_ICMPLT,IF_ICMPGE,IF_ICMPGT,IF_ICMPLE,IF_ACMPEQ,IF_ACMPNE,GOTO,JSR,RET,TABLESWITCH,LOOKUPSWITCH,IRETURN,LRETURN,FRETURN,DRETURN,ARETURN,RETURN,GETSTATIC,PUTSTATIC,GETFIELD,PUTFIELD,INVOKEVIRTUAL,INVOKESPECIAL,INVOKESTATIC,INVOKEINTERFACE,INVOKEDYNAMIC,NEW,NEWARRAY,ANEWARRAY,ARRAYLENGTH,ATHROW,CHECKCAST,INSTANCEOF,MONITORENTER,MONITOREXIT,,MULTIANEWARRAY,IFNULL,IFNONNULL,";
        OPCODES = new String[200];
        int n = 0;
        int index;
        for (int n2 = 0; (index = s.indexOf(44, n2)) > 0; n2 = index + 1) {
            Printer.OPCODES[n++] = ((n2 + 1 == index) ? null : s.substring(n2, index));
        }
        String s2 = "T_BOOLEAN,T_CHAR,T_FLOAT,T_DOUBLE,T_BYTE,T_SHORT,T_INT,T_LONG,";
        TYPES = new String[12];
        int n3 = 0;
        int n4 = 4;
        int index2;
        while ((index2 = s2.indexOf(44, n3)) > 0) {
            Printer.TYPES[n4++] = s2.substring(n3, index2);
            n3 = index2 + 1;
        }
        String s3 = "H_GETFIELD,H_GETSTATIC,H_PUTFIELD,H_PUTSTATIC,H_INVOKEVIRTUAL,H_INVOKESTATIC,H_INVOKESPECIAL,H_NEWINVOKESPECIAL,H_INVOKEINTERFACE,";
        HANDLE_TAG = new String[10];
        int n5 = 0;
        int n6 = 1;
        int index3;
        while ((index3 = s3.indexOf(44, n5)) > 0) {
            Printer.HANDLE_TAG[n6++] = s3.substring(n5, index3);
            n5 = index3 + 1;
        }
    }
}
